@@ -24,7 +24,7 @@
 #' }
 #' @importFrom stats gaussian
 #' @export
-clusterperm.lmer <- function (formula,data=NULL,family=gaussian(),weights=NULL,offset=NULL,series.var=~0,buildmerControl=list(direction='order',crit='LRT',quiet=TRUE,ddf='lme4'),nperm=1000,type='regression',parallel=FALSE,progress='none') {
+clusterperm.lmer <- function (formula,data=NULL,family=gaussian(),weights=NULL,offset=NULL,series.var=~0,buildmerControl=list(direction='order',crit='LRT',quiet=TRUE),nperm=1000,type='regression',parallel=FALSE,progress='none') {
 	if (length(type) != 1 || !type %in% c('anova','regression')) {
 		stop("Invalid 'type' argument (specify one of 'anova' or 'regression')")
 	}
@@ -72,16 +72,6 @@ clusterperm.lmer <- function (formula,data=NULL,family=gaussian(),weights=NULL,o
 	if (is.function(family)) {
 		family <- family()
 	}
-	wrap <- function (t,fun,formula,data,family,timepoints,buildmerControl,nperm,type,verbose) {
-		errfun <- function (e) {
-			# error in permutation-test function, return an empty result for this timepoint
-			warning(e)
-			data.frame(Factor=NA,p=NA)
-		}
-		ix <- timepoints == t
-		data <- data[ix,]
-		model <- tryCatch(fun(t,formula,data,family,timepoints,buildmerControl,nperm,type,verbose),error=errfun)
-	}
 	if (has.series) {
 		if (isTRUE(parallel)) {
 			verbose <- progress != 'none'
@@ -96,6 +86,16 @@ clusterperm.lmer <- function (formula,data=NULL,family=gaussian(),weights=NULL,o
 		verbose <- if (is.logical(progress)) progress else progress != 'none'
 		progress <- 'none'
 	}
+	wrap <- function (t,fun,formula,data,family,timepoints,buildmerControl,nperm,type,verbose) {
+		errfun <- function (e) {
+			# error in permutation-test function, return an empty result for this timepoint
+			warning(e)
+			data.frame(Factor=NA,p=NA)
+		}
+		ix <- timepoints == t
+		data <- data[ix,]
+		tryCatch(fun(t,formula,data,family,timepoints,buildmerControl,nperm,type,verbose),error=errfun)
+	}
 	results <- plyr::alply(sort(unique(timepoints)),1,wrap,fit.buildmer,formula,data,family,timepoints,buildmerControl,nperm,type,verbose,.parallel=parallel,.progress=progress,.inform=FALSE)
 	terms <- lapply(results,`[[`,'terms')
 	perms <- lapply(results,`[[`,'perms')
@@ -112,8 +112,15 @@ clusterperm.lmer <- function (formula,data=NULL,family=gaussian(),weights=NULL,o
 		df.LRT <- max(sapply(this.factor,function (x) x$df),na.rm=TRUE) #these will all be the same (because they are the same model comparison and these are ndf), except possibly in cases of rank-deficiency, hence why max is correct
 		thresh <- stats::qchisq(.95,df.LRT)
 		samp   <- sapply(this.factor,function (x) c(x$LRT,x$perms)) #columns are time, rows are samples
-		p      <- apply(samp,2,function (x) sum(x[-1] >= x[1],na.rm=TRUE) / sum(!is.na(x)))
-		stat   <- if (has.series) permuco::compute_clustermass(samp,thresh,sum,'greater')$main else NA
+		p      <- apply(samp,2,function (x) if (sum(!is.na(x)) == 1) NA else mean(x[-1] >= x[1],na.rm=TRUE))
+		# see GH issue #4: computing the cluster-mass test will fail in some cases, e.g. when only the intercept is involved (which is not permuted)
+		stat   <- rep(NA,NCOL(samp)) #account for the possible failure case
+		if (has.series) {
+			cmass <- try(suppressWarnings(permuco::compute_clustermass(samp,thresh,sum,'greater'))$main,silent=TRUE)
+			if (!inherits(cmass,'try-error')) {
+				stat <- cmass #succeeded!
+			}
+		}
 		df[df$Factor == x,c('p','cluster_mass','p.cluster_mass','cluster')] <- c(p,stat)
 	}
 
@@ -133,9 +140,6 @@ fit.buildmer <- function (t,formula,data,family,timepoints,buildmerControl,nperm
 	if (is.null(buildmerControl$quiet)) {
 		buildmerControl$quiet <- TRUE
 	}
-	if (is.null(buildmerControl$ddf)) {
-		buildmerControl$ddf <- 'lme4'
-	}
 
 	# First, make sure we only work with the tabular representation of the formula
 	if (inherits(formula,'formula')) {
@@ -147,30 +151,41 @@ fit.buildmer <- function (t,formula,data,family,timepoints,buildmerControl,nperm
 	fixed <- is.na(formula$grouping)
 	terms <- stats::setNames(,formula$term[fixed])
 
+	# For the fixed effects, we need to normalize the term/factor names (GH issue #4)
+	# Don't try to be clever and set new.names <- old.names; we don't want to overwrite e.g. factors in the user data with model-matrix columns -- we don't know how they interact with the user's r.e. specification
+	formula.fx <- buildmer::build.formula(NULL,formula[fixed,])
+	mm <- stats::model.matrix(formula.fx,data)
 	if (type == 'regression') {
-		# Convert factor terms into individual parameters
-		env <- environment(formula)
-		for (term in terms) {
-			if (is.factor(data[[term]])) {
-				X <- stats::model.matrix(stats::as.formula(paste0('~',term)),data)[,-1,drop=FALSE]
-				if (is.vector(X)) {
-					next #only one contrast
-				}
-				formula <- formula[!(formula$term == term & fixed),]
-				colnames(X) <- paste0(term,'_',colnames(X))
-				if (length(bad <- intersect(colnames(X),colnames(data)))) {
-					stop('Please rename the columns ',paste(bad,collapse=', '),' in your data')
-				}
-				for (x in colnames(X)) {
-					data[[x]] <- X[,x] #we can't use cbind as that will drop contrasts
-				}
-				formula <- rbind(formula,data.frame(index=NA,grouping=NA,term=colnames(X),code=unname(term),block=unname(term)))
-				fixed <- is.na(formula$grouping)
-			}
+		# one column per coefficient
+		old.names <- colnames(mm)
+		#terms <- apply(mm,2,identity,simplify=FALSE) #R >=4.1
+		terms <- lapply(1:NCOL(mm),function (i) mm[,i])
+	} else {
+		# multiple columns per coefficient
+		assign <- attr(mm,'assign')
+		tl <- attr(terms(formula.fx),'term.labels')
+		if (0 %in% assign) {
+			tl <- c('(Intercept)',tl)
+			assign <- assign + 1
 		}
-		environment(formula) <- env
-		terms <- stats::setNames(,formula$term[fixed])
+		old.names <- tl
+		terms <- lapply(1:length(tl),function (i) {
+			cols <- mm[,assign == i,drop=FALSE]
+			colnames(cols) <- rep('',ncol(cols))
+			cols
+		})
 	}
+	new.names <- paste0('X',1:length(old.names))
+	for (i in 1:length(new.names)) {
+		if (new.names[i] %in% names(data)) {
+			stop('Please remove column ',new.names[i],' from your data; its name conflicts with a name used internally in permutes!')
+		}
+		data[[new.names[i]]] <- terms[[i]]
+	}
+	new.formula <- data.frame(index=NA,grouping=NA,term=new.names,code=new.names,block=new.names)
+	e <- environment(formula)
+	formula <- rbind(new.formula,formula[!fixed,])
+	environment(formula) <- e
 
 	.weights <- data$.weights; .offset <- data$.offset #silence R CMD check warning; also necessary for buildmer =2.0, which did not support NSE for these
 	if (utils::packageVersion('buildmer') < '2.0') {
@@ -179,10 +194,11 @@ fit.buildmer <- function (t,formula,data,family,timepoints,buildmerControl,nperm
 		buildmerControl$args <- c(buildmerControl$args,list(weights=.weights,offset=.offset))
 		bm <- buildmer::buildmer(formula=formula,data=data,family=family,buildmerControl=buildmerControl)
 	}
-	perms <- lapply(terms,function (term) {
+
+	perms <- lapply(1:length(new.names),function (i) {
 		if (verbose) {
 			time <- Sys.time()
-			nmodels <- length(terms) * length(unique(timepoints))
+			nmodels <- length(new.names) * length(unique(timepoints))
 		}
 
 		# https://www.ncbi.nlm.nih.gov/pmc/articles/PMC3883440/ propose that, to test a random effect:
@@ -200,7 +216,7 @@ fit.buildmer <- function (t,formula,data,family,timepoints,buildmerControl,nperm
 
 		# 1. Get the marginal errors based on quantities from the alternative model
 		# These are y - XB - Zu, with the effect of interest *removed* from X
-		# It's easier for us to just take the residuals and add this effect back in, but we need to figure out its name...
+		# It's easier for us to just take the residuals and add this effect back in
 		if (inherits(bm@model,'merMod')) {
 			X <- lme4::getME(bm@model,'X')
 			B <- lme4::fixef(bm@model)
@@ -208,57 +224,38 @@ fit.buildmer <- function (t,formula,data,family,timepoints,buildmerControl,nperm
 			X <- stats::model.matrix(formula(bm@model),data)
 			B <- stats::coef(bm@model)
 		}
-		if (any(i <- !is.finite(B))) {
-			B[i] <- 0
-		}
-		if (term == '1') {
-			# If it's the intercept, things are simple
-			X[colnames(X) != '(Intercept)'] <- 0
-			e <- stats::resid(bm@model) + X %*% B
-			X <- X[,colnames(X) == '(Intercept)']
-		} else {
-			# If it's not the intercept, things are more complicated: we need to figure out the name in the model matrix
-			tab.full <- formula[is.na(formula$grouping),]
-			formula.full <- buildmer::build.formula(NULL,tab.full)
-			X.full <- stats::model.matrix(formula.full,data)
-			tab.restricted <- formula[formula$term != term & is.na(formula$grouping),]
-			formula.restricted <- buildmer::build.formula(NULL,tab.restricted)
-			X.restricted <- stats::model.matrix(formula.restricted,data)
-			if (!(NCOL(X.full) > NCOL(X.restricted))) {
-				return(list(perms=0*1:nperm,LRT=0,df=0))
-			}
-			normalized.colnames <- function (X) { #because interaction terms may have been wickedly reordered between colnames(X) and colnames(X.restricted)
-				split <- strsplit(colnames(X),':')
-				norm <- lapply(split,sort)
-				sapply(norm,paste0,collapse=':')
-			}
-			stopifnot(all(normalized.colnames(X.restricted) %in% normalized.colnames(X)))
-			want <- !normalized.colnames(X) %in% normalized.colnames(X.restricted)
-			X[,!want] <- 0
-			e <- stats::resid(bm@model) + X %*% B
-			X <- X[,want]
+		keep <- colnames(X) == new.names[i]
+		X[,!keep] <- 0 #must modify X rather than B because X will be used as predictors in the permuted models
+		if (any(wh <- !is.finite(B))) { #rank-deficiency in lm
+			X[,wh] <- 0
 		}
+		e <- stats::resid(bm@model) + X %*% B
 
 		# 2/3. Random effects have already been partialed out, so these are independent and exchangeable
 		# 4/5. Permute them and estimate a null and alternative model on the permuted data
 		# The offset has been partialed out already, so will be ignored
 		fit <- if (bm@p$is.gaussian) function (formula,data) stats::lm(formula,data,weights=.weights) else function (formula,data) suppressWarnings(stats::glm(formula,family=family,data=data,weights=.weights))
-		perms <- lapply(1:nperm,function (i) try({
-			s <- sample(seq_along(e))
-			data <- list(
-				y = family$linkinv(e[s]),
-				X = X,
-				.weights = data$.weights[s]
-			)
-			m1 <- fit(y ~ 0+X,data)
-			m0 <- fit(y ~ 0,data)
-			as.numeric(2*(stats::logLik(m1)-stats::logLik(m0)))
-		},silent=TRUE))
-		bad <- sapply(perms,inherits,'try-error')
-		if (any(bad)) {
-			perms[bad] <- NA
+		# Optimization: nothing to do if the actually-kept columns are constant
+		if (length(unique(as.vector(X[,keep]))) == 1) {
+			perms <- NA
+		} else {
+			perms <- lapply(1:nperm,function (i) try({
+				s <- sample(seq_along(e))
+				data <- list(
+					y = family$linkinv(e[s]),
+					X = X,
+					.weights = data$.weights[s]
+				)
+				m1 <- fit(y ~ 0+X,data)
+				m0 <- fit(y ~ 0,data)
+				as.numeric(2*(stats::logLik(m1)-stats::logLik(m0)))
+			},silent=TRUE))
+			bad <- sapply(perms,inherits,'try-error')
+			if (any(bad)) {
+				perms[bad] <- NA
+			}
+			perms <- unlist(perms)
 		}
-		perms <- unlist(perms)
 
 		# Wrap up
 		data$y <- family$linkinv(e)
@@ -274,42 +271,30 @@ fit.buildmer <- function (t,formula,data,family,timepoints,buildmerControl,nperm
 		list(perms=perms,LRT=LRT,df=df)
 	})
 	LRT <- sapply(perms,`[[`,'LRT')
+	names(LRT) <- new.names
+
 	scale.est <- !family(bm@model)$family %in% c('binomial','poisson')
 	is.mer <- inherits(bm@model,'merMod')
 	if (type == 'regression') {
-		se <- sqrt(diag(as.matrix(stats::vcov(bm@model)))) #as.matrix needed to work around 'Error in diag(vcov(bm@model)) : long vectors not supported yet: array.c:2186'
 		if (inherits(bm@model,'merMod')) {
 			beta <- lme4::fixef(bm@model)
-			if (length(beta) < length(terms)) { #rank-deficiency
-				missing <- setdiff(names(terms),names(beta))
-				names(se) <- names(beta)
-				beta[missing] <- NA
-				se[missing] <- NA
-				beta <- beta[names(terms)]
-				se <- se[names(terms)]
-			}
 		} else {
 			beta <- stats::coef(bm@model)
 		}
-		tname <- if (scale.est) 't' else 'z'
-		df <- data.frame(Factor=unname(terms),LRT=unname(LRT),beta=unname(beta),t=unname(beta/se))
-		colnames(df)[4] <- tname
-		list(terms=terms,perms=perms,df=df)
+		se    <- sqrt(diag(as.matrix(stats::vcov(bm@model)))) #as.matrix needed to work around 'Error in diag(vcov(bm@model)) : long vectors not supported yet: array.c:2186'
+		LRT   <- LRT[new.names]
+		beta  <- beta[new.names]
+		se    <- se[new.names]
+		df    <- data.frame(Factor=old.names,LRT=unname(LRT),beta=unname(beta),t=unname(beta/se))
+		colnames(df)[4] <- if (scale.est) 't' else 'z'
+		list(terms=old.names,perms=perms,df=df)
 	} else {
 		if (inherits(bm@model,'gam')) {
 			anovatab <- stats::anova(bm@model) #is Type III
 			Fvals <- anovatab$pTerms.chi.sq / anovatab$pTerms.df
 			Fname <- 'F'
 			df <- anovatab$pTerms.df
-			# gam anova removes the intercept; restore it if needed
-			if (names(terms)[1] == '1') {
-				Fval1 <- anovatab$p.t['(Intercept)']^2
-				Fvals <- c(Fval1,Fvals)
-				df <- c(1,df)
-				names(Fvals) <- c('1',rownames(anovatab$pTerms.table))
-			} else {
-				names(Fvals) <- rownames(anovatab$pTerms.table)
-			}
+			names(df) <- names(Fvals) <- rownames(anovatab$pTerms.table)
 		} else {
 			if (is.mer) {
 				anovatab <- car::Anova(bm@model,type=3,test='Chisq')
@@ -336,15 +321,13 @@ fit.buildmer <- function (t,formula,data,family,timepoints,buildmerControl,nperm
 				}
 			}
 			df <- anovatab$Df
-			names(Fvals) <- rownames(anovatab)
+			names(df) <- names(Fvals) <- rownames(anovatab)
 		}
-		if (length(Fvals) < length(terms)) { #rank-deficiency
-			missing <- setdiff(names(terms),names(Fvals))
-			Fvals[missing] <- NA
-			Fvals <- Fvals[names(terms)]
-		}
-		df <- data.frame(Factor=unname(terms),df=df,LRT=unname(LRT),F=unname(Fvals))
+		df    <- df[new.names]
+		LRT   <- LRT[new.names]
+		Fvals <- Fvals[new.names]
+		df <- data.frame(Factor=old.names,df=df,LRT=unname(LRT),F=unname(Fvals))
 		colnames(df)[4] <- Fname
-		list(terms=terms,perms=perms,df=df)
+		list(terms=old.names,perms=perms,df=df)
 	}
 }
